program testing

implicit none

! general declarations
integer :: t1, t2
integer :: Nitem, Nrestr
real, dimension(:), allocatable :: pi
real, dimension(:,:), allocatable :: R
real, dimension(:), allocatable :: Rc
integer, dimension(:), allocatable :: x

! sampling declarations
integer :: Niter, trestr, tpi
real :: c, crestr, f, small, BF, P, Q, feval(1)
real, dimension(:), allocatable :: eval
integer, parameter :: Nf=1
real, dimension(:), allocatable :: initpi

! constrained sampling declarations
real, dimension(:), allocatable :: R1,pimin,Rcmin,lowup
real, dimension(:,:), allocatable :: Rrest
real :: low, up

! declarations for sd of BF
real, dimension(:), allocatable :: chit, fhit, csamp, fsamp
real, dimension(:) :: RR(1), BFu(100000), BFc(100000), sorted(100000),ftot(100000),ctot(100000)
integer :: nr
real :: pin, qin, Lu, Uu, Lc, Uc

! open de invoer file en de data file
open(21,file='input.txt')
open(23,file='output.txt')

! lees het aantal items en aantal restricties van de invoer file
read(21,*)
read(21,*) Nitem, Nrestr, Niter

! zet de random number generator

call rnset(Nf)

! alloceer
allocate(pi(Nitem), x(Nitem), R(Nrestr,Nitem), Rc(Nrestr), eval(Nrestr),initpi(Nitem),pimin(Nitem-1))
allocate(chit(Nrestr),fhit(Nrestr),csamp(Nrestr),fsamp(Nrestr))

! lees de restricities van de restrictie file
read(21,*)
do t1=1,Nrestr
read(21,*) R(t1,1:Nitem), Rc(t1)
end do
read(21,*)
read(21,*) initpi
! lees de data van de data file
read(21,*)
read(21,*) x

! bereken de complexiteit mbv de decompositie

c=1

!write(23,*)"decomposed complexity values" 

do trestr=1,Nrestr

crestr=0

! sample uit een Nitem dimensionele uniforme verdeling gebruikmakend van de trestr-1 voorgaande restricties
! dit wordt gedaan met behulp van een Gibbs sampler

! assign starting values

pi = initpi

do t1=1,Niter

do tpi=1,Nitem 

! determine lower and upper bound for pi(tpi) using the trestr-1 previous restrictions

if (trestr==1) then

low=0
up=1

else

allocate(R1(trestr-1),Rrest(trestr-1,Nitem-1),Rcmin(trestr-1),lowup(trestr-1))
R1=R(1:trestr-1,tpi)
Rrest(1:trestr-1,1:tpi-1)=R(1:trestr-1,1:tpi-1)
Rrest(1:trestr-1,tpi:Nitem-1)=R(1:trestr-1,tpi+1:Nitem)
pimin(1:tpi-1)=pi(1:tpi-1)
pimin(tpi:Nitem-1)=pi(tpi+1:Nitem)
Rcmin(1:trestr-1)=Rc(1:trestr-1)

lowup=(Rcmin-matmul(Rrest,pimin))/abs(R1)

low=0
up=1

do t2=1,trestr-1

if (R1(t2) > 0) low=max(low,lowup(t2))
if (R1(t2) < 0) up=min(up,-1 * lowup(t2))

end do

deallocate(R1,Rrest,Rcmin,lowup)

end if

! sample from a restricted uniform distribution

CALL RNUN (Nf, feval) 

feval(1)=feval(1)*(up-low)

pi(tpi)=feval(1)+low

end do ! do tpi=1,Nitem

! bereken c door de trestr huidige restricties na te lopen
eval = matmul(R,pi)-Rc

! hieronder small laten berekenen over de trestr actuele elementen van small
small = minval(eval(1:trestr))
 
if  (small > 0) crestr=crestr+1.0/Niter

end do ! do t1=1,Niter

!write(23,'(I2,2x,f6.4)') trestr, crestr

write(*,*) "Completed: Evaluation of Complexity Restriction  ", trestr

chit(trestr) = crestr * Niter

c = c * crestr

end do ! do trestr=1,Nrestr

!write(23,*)


! bereken de fit mbv de decompositie

!write(23,*)"decomposed fit values" 

f=1

do trestr=1,Nrestr

crestr=0

! sample uit een Nitem dimensionele uniforme verdeling gebruikmakend van de trestr-1 voorgaande restricties
! dit wordt gedaan met behulp van een Gibbs sampler

! assign starting values

pi=initpi

do t1=1,Niter

do tpi=1,Nitem 

! determine lower and upper bound for pi(tpi) using the trestr=1 previous restrictions

if (trestr==1) then

low=0
up=1

else

allocate(R1(trestr-1),Rrest(trestr-1,Nitem-1),Rcmin(trestr-1),lowup(trestr-1))
R1=R(1:trestr-1,tpi)
Rrest(1:trestr-1,1:tpi-1)=R(1:trestr-1,1:tpi-1)
Rrest(1:trestr-1,tpi:Nitem-1)=R(1:trestr-1,tpi+1:Nitem)
pimin(1:tpi-1)=pi(1:tpi-1)
pimin(tpi:Nitem-1)=pi(tpi+1:Nitem)
Rcmin(1:trestr-1)=Rc(1:trestr-1)

lowup=(Rcmin-matmul(Rrest,pimin))/abs(R1)

low=0
up=1

do t2=1,trestr-1

if (R1(t2) > 0) low=max(low,lowup(t2))
if (R1(t2) < 0) up=min(up,-1 * lowup(t2))

end do

deallocate(R1,Rrest,Rcmin,lowup)

end if

! sample uit een beta verdeling met lower bound low en upper bound up 

CALL RNUN (Nf, feval) 
if (x(tpi)==1) pi(tpi) = sqrt(low**2 + feval(1) * (up**2 - low**2))

if (x(tpi)==0) pi(tpi)= 1 - .5 * sqrt(4 - 4 * (2*low - low**2 + feval(1) * (low**2 - 2 * low + 2 * up - up ** 2)))



end do ! do tpi=1,Nitem

! bereken c door de trestr huidige restricties na te lopen
eval = matmul(R,pi)-Rc

! hieronder small laten berekenen over de trestr actuele elementen van small
small = minval(eval(1:trestr))
if  (small > 0) crestr=crestr+1.0/Niter

end do ! do t1=1,Niter

!write(23,'(I2,2x,f6.4)') trestr, crestr

write(*,*) "Completed: Evaluation of Fit Restriction  ", trestr

fhit(trestr) = crestr * Niter
f = f * crestr

end do ! do trestr=1,Nrestr

!write(23,*)

! bereken de Bayes factor
BF = f/c

! bereken de sd van de Bayes factor, zowel voor BFmu als voor BFmc
nr = 1
do t1 =1,100000

do t2 =1,Nrestr
pin = chit(t2) + 1
qin = Niter - chit(t2) + 1
call rnbet(nr,pin,qin,rr)
csamp(t2)=rr(1)
pin = fhit(t2) + 1
qin = Niter - fhit(t2) + 1
call rnbet(nr,pin,qin,rr)
fsamp(t2)=rr(1)
end do

ftot(t1)=1
ctot(t1)=1
do t2 = 1,Nrestr
ftot(t1) = ftot(t1) * fsamp(t2)
ctot(t1) = ctot(t1) * csamp(t2)
end do

end do ! t1=1,100000

BFu = ftot/ctot
BFc = BFu / ((1.0-ftot)/(1.0-ctot))

! sorteer BFu en BFc en bepaal het 2.5 en 97.5 percentiel
nr=100000
call svrgn(nr,BFu,sorted)
Lu = sorted(2500)
Uu = sorted (97500)
call svrgn(nr,BFc,sorted)
Lc = sorted(2500)
Uc = sorted (97500)

! schrijf de resultaten naar een output file
write(23,*) "fit ="
write(23,'(f12.10,)') f
write(23,*)
write(23,*) "complexity ="
write(23,'(f12.10)') c
write(23,*)
write(23,*) "Bayes factor for hypothesis versus unconstrained alternative ="
write(23,'(f17.10)') bf
write(23,*) "interval for Bayes factor due to estimation error ="
write(23,'(f17.10,3x,f17.10)') Lu, Uu
write(23,*)
write(23,*) "Bayes factor for hypothesis versus complement ="
bf = bf /((1.0-f)/(1.0-c))
write(23,'(f17.10)') bf
write(23,*) "interval for Bayes factor due to estimation error ="
write(23,'(f17.10,3x,f17.10)') Lc, Uc

write(*,*) "Analysis Completed"

end program testing





